﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.SymbolicExpressionTreeEncoding {
  [Item("Fragment", "Class that represents a tree fragment")]
  [StorableClass]
  public class Fragment : Item, IFragment {
    [Storable]
    private ISymbolicExpressionTreeNode root;
    public ISymbolicExpressionTreeNode Root { get { return root; } set { root = value; } }
    [Storable]
    private int index = -1;
    public int Index { get { return index; } set { index = value; } }
    [Storable]
    private int oldIndex = -1;
    public int OldIndex { get { return oldIndex; } set { oldIndex = value; } }

    private int length = -1;
    public int Length {
      get {
        if (Root == null) return 0;
        if (length == -1) length = Root.GetLength();
        return length;
      }
      set { length = value; }
    }

    [StorableConstructor]
    private Fragment(bool serializing) : base(serializing) { }
    private Fragment(Fragment original, Cloner cloner)
      : base(original, cloner) {
      // avoid deep copy of the symbolic expression tree node
      Root = original.Root;
      Length = original.Length;
      Index = original.Index;
    }

    public Fragment() { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new Fragment(this, cloner);
    }
  }
}